/* Group3Player300.java - A straightforward implementation of Problem 3
 * of Programming/Problem Solving, Fall, 2002
 * v1.0: FINAL RELEASE
 *       Changed the split army strategy to make both sides march horizontally
 *       Ensures program runs in extreme cases (i.e. 200 robots, side=50)
 *       NOTE: Could not test for n > 100 robots; the interface wouldn't allow the input
 *       For less than 5 robots:
 *       n = 1: Will continue to use "rose" algorithm, with fixes
 *       n = 2: Will try a diagonal square-building algorithm (see below)
 *       BUG: This algorithm only works for the first two rows, then gets wacky.
 *       n = 3: Same as 2, but with an extra "ambitious" robot trying to get the 
 *              bottom portion of the board (bot. 1/3 for high player games, bot. 1/2 otherwise
 *       n = 4: Same as 3, but one more "ambitious" robot aiming for the top portion.
 * v0.3: Splits army up in two, but only for 2 or 3 player game
 *       Half of army marches horizontally, half vertically
 *       Will use William's strategy for n < 5
 *       Also will consider an implementation for n > size of board
 * v0.2: Implements a strategy (to be announced) for n<5 robots, randomizes
 *       the corner to start from 
 * v0.1: Player lines up in the top left corner, oriented vertically, and
 *       simply marches horizontally across the game board
 *       Implementation guarantees a new rectangle every two moves when 
 *       interrupted, provided there isn't a chaser robot or a large claimed 
 *       area
 *
 * Written by: Noel Vega (noel.vega@pba.com)
 */

package Rectangles;

import ui.*;
import java.util.*;
import java.io.*;
import java.awt.Color;

public final class OldGroup3Player300 implements IFCPlayer {

    // Constants for game engine operation   
    final Color  _CCOLOR = new Color(0.0f, 0.0f, 0.8f);  // a shade of blue
    final boolean DEBUG = false;                         // for surpressing debugging output     
    final boolean DEBUG2 = false;                        // more of the same
    final boolean DEBUG3 = false;                         // ... and some more.
    
    // Constants for player operation
    static final int TOPRIGHT = 0;            // Start at the top right corner
    static final int TOPLEFT = 1;             // Start at the top left corner
    static final int BOTRIGHT = 2;            // Start at the bottom right corner
    static final int BOTLEFT = 3;             // Start at the bottom left corner
    static final int LOW = 4;                 // what's a "low" number of robots?
    static final int MEDIUM = 1000;           // anything above this is "high" 
    static final int RECT_SIZE = 5;           // size of rectangle to build towards
    static final int ROBTHRESH = 10;          // When to split the army...
    static final double UPPERTHRESH = 0.66;   // ... and when not to.
    
    
    // Global vars
    String name;                 // name variable
    Rectangles _rect;              // game variable
    static Random _random;         // Mostly used in version 0.1
    OldG3Point[] positions;   
    char[] directions;             // for remembering what directions to move in
    char vertTendency;             // remember which vertical direction we're moving in
    char vertTendency2;            // Do the same for the other half of the army
    char turnTendency = _CWEST;    // remember how we were turning (for strat. 1)
    char leftBotTendency = _CEAST; // for strat. 1: Determine which direction left bot is building
    boolean endMarch = false;      // Know when to turn around
    boolean wrongway = false;      // for strat. 1: Know when we are "marching backwards"
    int roundNum;
    int numVertical;    // remember how many consec. moves were vertical
    int numVertical2;   // same for second portion of the army (for strat. 3)
    int numRobots;
    int consecutive;    // number of consec moves in same direction (for strat. 1)
    int sides = 0;      // number of completed sides in an attempted rect (for strat. 1)
    int size;           // game board size
    int cornerPicked;   // What corner do we start at?
    int numPlayers;     // How many players are in the game?
    int southBoundary;  // For strat. 1: Southern boundary of building
    int midpoint;       // declaring half of the army
    int numRoses = 0;       // For strat. 1, 1 robot: How many iterations?
    int newRoseThresh = 7;  // For strat. 1, 1 robot: How far before starting a new rose?
    double robotRatio;  // Ratio of robots to game board size
    
    
    // Methods 
       
    // Register the player with the game engine, and initialize local vars
    public Robot[] register(Rectangles __rectangles) throws Exception {
	
        Robot[] returnVals;
        _rect = __rectangles;
	roundNum = _rect.rounds();
	numVertical = 0;

	// Initialize the randomizer - but ONLY if it's necessary!
        if (_random  == null) {
            _random = new Random();
        }

        // initialize everything else
	numPlayers = _rect.numPlayers();
	numRobots = _rect.numRobots();
        size = _rect.size();
        returnVals = new Robot[numRobots];
	directions = new char[numRobots];
	positions = new OldG3Point[numRobots];
        
	if (numRobots > 1) {
	    robotRatio = (double)(numRobots / size);
	    midpoint = numRobots / 2;
	    if ((numRobots % midpoint) != 0)
		midpoint++;
	}
        // Now we decide the placement
	// For numRobots <= LOW, use William's strategy
	// Otherwise, proceed
	if (numRobots <= LOW) {
	    cornerPicked = _random.nextInt(4);
	    returnVals = strategy1_init(cornerPicked);  // William's strat. will go here
	    name = "Spiralling Army Cadets";  // renaming the player
	}
	// Now, the army setup
	// If there's only 2 or 3 players, and at least robThresh robots,
	// split the army up and march at 90 degree angles
        // Do same for numRobots > size
	// Otherwise, use the traditional army algorithm
	else if (((numPlayers > 3) && (numRobots < size)) || (numRobots < 10)) {
	    cornerPicked = _random.nextInt(4);  // Random corner to start with
	    returnVals = strategy2_init(cornerPicked);
	    name = "Army of Players";      // again, renaming
	}
	else if (numRobots < size) {
	    // NEW Split Army Strategy:
	    // Pick a corner. Put half the army there, as always.
	    // Line up the second half of the army horizontally, along the 
	    // diagonally opposite corner
	    // First half of army marches horizontally, as always
	    // Second half marches vertically
	    cornerPicked = _random.nextInt(4);  // Again, random corner
	    returnVals = strategy3_init(cornerPicked);
	    name = "Split Army Player";    // final rename
	}
	else {
	    // New addition for numRobots >= size
	    // Lines up on opposite sides, marches together
	    cornerPicked = _random.nextInt(2);  // A random SIDE this time
	    if (cornerPicked == 0)
		returnVals = strategy4_init(_CWEST);
	    else returnVals = strategy4_init(_CEAST);
	    name = "Army BLITZ";    // No, THIS is the last rename...
	}

	// And here we make our move.
	return returnVals;
    }

    // Strategy to use for few robots
    private Robot[] strategy1_init(int corner) throws Exception {
	Robot[] returns = new Robot[numRobots];
        switch (numRobots) {
	case 1: 
	    returns[0] = new Robot(16, 4);  // 4 away from corner
	    positions[0] = new OldG3Point(16, 4);
	    break;
	case 2:
	    returns[0] = new Robot(0, size - 1);
	    positions[0] = new OldG3Point(0, size - 1);
	    returns[1] = new Robot(size - 1, size - 1);
	    positions[1] = new OldG3Point(size - 1, size - 1);
	    southBoundary = size - 1;
	    break;
	case 3:
	    returns[0] = new Robot(0, size - 1);
	    positions[0] = new OldG3Point(0, size - 1);
	    returns[1] = new Robot(size - 1, size - 1);
	    positions[1] = new OldG3Point(size - 1, size - 1);
	    returns[2] = new Robot(0, size - 1);
	    positions[2] = new OldG3Point(0, size - 1);
	    southBoundary = size - 1;
	    break;
	case 4:
	    returns[0] = new Robot(0, size - 1);
	    positions[0] = new OldG3Point(0, size - 1);
	    returns[1] = new Robot(size - 1, size - 1);
	    positions[1] = new OldG3Point(size - 1, size - 1);
	    returns[2] = new Robot(0, size - 1);
	    positions[2] = new OldG3Point(0, size - 1);
	    returns[3] = new Robot(0, ((2 * size) / 3) - 1);
	    positions[3] = new OldG3Point(0, ((2 * size) / 3) - 1);
	    southBoundary = size - 1;
	}
	
	return returns;
    }
	
    // Strategy for a moderate number of robots
    // AKA: The "Army" algorithm
    private Robot[] strategy2_init(int corner) throws Exception {
	Robot[] returns = new Robot[numRobots];

	// The idea: Line up the robots, let them march together
	// As they march, a continuous rectangle is formed
	// If the rectangle is broken, a new one is easily formed in two moves!
	if (DEBUG)
	    System.out.println("Corner picked: " + corner);
	if (corner == TOPLEFT) {
	    vertTendency = _CSOUTH;
	    for (int i = 0; i < numRobots; i++) {
		returns[i] = new Robot(0, i);  // the domination starts in the corner!
		directions[i] = _CEAST;
		positions[i] = new OldG3Point(0, i);
	    }
	}
	else if (corner == TOPRIGHT) {
	    vertTendency = _CSOUTH;
	    for (int i = 0; i < numRobots; i++) {
		returns[i] = new Robot(size - 1, i);
		directions[i] = _CWEST;
		positions[i] = new OldG3Point(size - 1, i);
	    }
	}
	else if (corner == BOTLEFT) {
	    vertTendency = _CNORTH;
	    for (int i = 0; i < numRobots; i++) {
		returns[i] = new Robot(0, (size - (numRobots - i)));
		directions[i] = _CEAST;
		positions[i] = new OldG3Point(0, size - (numRobots - i));
	    }
	}
	else {   // corner == BOT_RIGHT
	    vertTendency = _CNORTH;
	    for (int i = 0; i < numRobots; i++) {
		returns[i] = new Robot(size - 1, size - (numRobots - i));
		directions[i] = _CWEST;
		positions[i] = new OldG3Point(size - 1, size - (numRobots - i));
	    }
	}
	return returns;
    }

    // The two or three player split army strategy
    // Line up the first half of the army along a corner, as always
    // CHANGED: The second half now marches horizontally, like the first
        
    private Robot[] strategy3_init(int corner) throws Exception {
	Robot[] returns = new Robot[numRobots];
	int center = size / 2;
	// Make sure the horizontal section gets the extra robot,
	// in the case of an odd number of robots
	if (DEBUG)
	    System.out.println("Corner picked: " + corner);
	if (corner == TOPLEFT) {
	    vertTendency = _CSOUTH;
	    vertTendency2 = _CNORTH;
	    for (int i = 0; i < midpoint; i++) {
		returns[i] = new Robot(0, i);  // the domination starts in the corner!
		directions[i] = _CEAST;
		positions[i] = new OldG3Point(0, i);
	    }
	    for (int j = midpoint; j < numRobots; j++) {
		returns[j] = new Robot((size - 1), size - (numRobots - j));
		directions[j] = _CWEST;
		positions[j] = new OldG3Point((size - 1), size - (numRobots - j));
	    }
	}
	else if (corner == TOPRIGHT) {
	    vertTendency = _CSOUTH;
	    vertTendency2 = _CNORTH;
	    for (int i = 0; i < midpoint; i++) {
		returns[i] = new Robot(size - 1, i);
		directions[i] = _CWEST;
		positions[i] = new OldG3Point(size - 1, i);
	    }
	    for (int j = midpoint; j < numRobots; j++) {
		returns[j] = new Robot(0, size - (numRobots - j));
		directions[j] = _CEAST;
		positions[j] = new OldG3Point(0, size - (numRobots - j));
	    }
	}
	else if (corner == BOTLEFT) {
	    vertTendency = _CNORTH;
	    vertTendency2 = _CSOUTH;
	    for (int i = 0; i < midpoint; i++) {
		returns[i] = new Robot(0, (size - (midpoint - i)));
		directions[i] = _CEAST;
		positions[i] = new OldG3Point(0, size - (midpoint - i));
	    }
	    for (int j = midpoint; j < numRobots; j++) {
		returns[j] = new Robot((size - 1), (j - midpoint));
		directions[j] = _CWEST;
		positions[j] = new OldG3Point((size - 1), (j - midpoint));
	    }
	}
	else {   // corner == BOT_RIGHT
	    vertTendency = _CNORTH;
	    vertTendency2 = _CSOUTH;
	    for (int i = 0; i < midpoint; i++) {
		returns[i] = new Robot(size - 1, size - (midpoint - i));
		directions[i] = _CWEST;
		positions[i] = new OldG3Point(size - 1, size - (midpoint - i));
	    }
	    for (int j = midpoint; j < numRobots; j++) {
		returns[j] = new Robot(0, (j - midpoint)); 
		directions[j] = _CEAST;
		positions[j] = new OldG3Point(0, (j - midpoint)); 
	    }
	}
	return returns;    
	
    }

    // returns a 90 degree move right
    private char move90Right(char direction) {
	if (direction == _CEAST)
	    return _CSOUTH;
	else if (direction == _CSOUTH)
	    return _CWEST;
	else if (direction == _CWEST) 
	    return _CNORTH;
	else return _CEAST;
    }

    // returns a 90 degree move left
    private char move90Left(char direction) {
	if (direction == _CEAST) 
	    return _CNORTH;
	else if (direction == _CNORTH)
	    return _CWEST;
	else if (direction == _CWEST)
	    return _CSOUTH;
	else return _CEAST;
    }
    
    // returns a 180 degree turn
    private char move180(char direction) {
	if (direction == _CEAST)
	    return _CWEST;
	else if (direction == _CWEST) 
	    return _CEAST;
	else if (direction == _CNORTH)
	    return _CSOUTH;
	else return _CNORTH;
    }

    // Strategy for numRobots > size
    // Lines up on different sizes, depending on exactly how many robots we're talking about...
    // numRobots == size: all on one side
    // size < numRobots <= 2 * size: opposite sides
    // 2 * size < numRobots <= 4 * size: Form a rectangle: long sides are of size size,
    //                                   short sides are 1/2 of the remainder
    // 4 * size < numRobots: Just line them all up on the border and stay.
    //                       When the other teams' robots clear, we get the whole board.
    private Robot[] strategy4_init(char side)  throws Exception {
	Robot[] returns = new Robot[numRobots];
	int chosenx;      // x coordinate of selected side
	int nonChosenx;   // the opposite side
	char direction;   // direction to march in initially
	// First, we need to translate the chosen side into an x coordinate
	if (side == _CEAST) {
	    chosenx = (size - 1);
	    nonChosenx = 0;
	    direction = _CWEST;
	}
	else {
	    chosenx = 0;
	    nonChosenx = size - 1;
	    direction = _CEAST;
	}
	
	// No matter what the case is, we will be lining up part of the robots on one side.
	for (int i = 0; i < size; i++) {
	    if (DEBUG3)
		System.out.println("Attempting to place a robot at: x = " + chosenx + ", y = " + i);
	    returns[i] = new Robot(chosenx, i);
	    directions[i] = direction;
	    positions[i] = new OldG3Point(chosenx, i);
	}
	
	// Now that that's out of the way...
	// For numRobots == size, we're done.
	// Now, for up to twice the size of a side of a board...
	if (numRobots <= 2 * size) {
	    int j = 0;
	    for (int i = size; i < numRobots; i++) {
		returns[i] = new Robot(nonChosenx, j);
		directions[i] = move180(direction);
		positions[i] = new OldG3Point(nonChosenx, j);
		j++;
		if (DEBUG3)
		    System.out.println("Attempting to place a robot at x=" + nonChosenx
				       + ", y = " + j);
	    }
	}

	// Now for numRobots between 2 * size and 4 * size
	// The idea here is to start out with a large rectangle already formed, 
	// and sweep across the board with that rectangle.
	else if ((numRobots > 2 * size) && (numRobots < 4 * size)) {
	    int j = 0;
	    int i = size;
	    int overflow = numRobots - (2 * size);
	    if (chosenx == 0)
		nonChosenx = (overflow / 2) - 1;
	    else nonChosenx = chosenx - (overflow / 2) - 1;
	    if (DEBUG3) {
		System.out.println("nonChosenx = " + nonChosenx);
		System.out.println("size = " + size);
		System.out.println("overflow = " + overflow);
		System.out.println("chosenx = " + chosenx);
	    }
	    for (; i < size * 2; i++) {
		returns[i] = new Robot(nonChosenx, j);
		if (DEBUG3) {
		    
		    System.out.println("Trying to place at x = " + nonChosenx + " y = " + j);
		    if (direction == _CEAST)
			System.out.println("Direction is east");
		    else System.out.println("Directions is west");
		}
		directions[i] = direction;
		positions[i] = new OldG3Point(nonChosenx, j);
		j++;
	    }
	    // Now we place the small ends of the rectangle
	    j = 0;
	    int currentx;
	    if (chosenx == 0)
		currentx = 0;
	    else currentx = chosenx - (overflow / 2);
	    for (; i < numRobots - (overflow / 2); i++) {
		returns[i] = new Robot(currentx, j);
		directions[i] = direction;
		positions[i] = new OldG3Point(currentx, j);
		currentx++;
	    }
	    
	    j = size - 1;
	    if (chosenx == 0)
		currentx = 0;
	    else currentx = chosenx - (overflow / 2);
	    for (; i < numRobots; i++) {
		returns[i] = new Robot(currentx, j);
		directions[i] = direction;
		positions[i] = new OldG3Point(currentx, j);
		currentx++;
	    }
	}    
       
	// Finally, the strategy for numRobots > size * 4
	// Note that we start in a square that surrounds the whole board here.
	else {
	    // For a 2 or 3 player game, let the bots stay put, so we eventually get everything.
	    // Otherwise, line up in a huge square, march to the other side.
	    int k = 0;
	    if (numPlayers <= 3) {
		for (int i = 0; i < numRobots; i++) {
		    directions[i] = _CSTAY;
		}
	    }
	    else {
		int l = 0;
		for (; l < size; l++) {
		    directions[l] = _CEAST;
		}
		for (; l < size * 2; l++) {
		    directions[l] = _CSOUTH;
		}
		for (; l < size * 3; l++) {
		    directions[l] = _CNORTH;
		}
		for (; l < size * 4; l++) {
		    directions[l] = _CWEST;
		}
		for (; l < numRobots; l++) {
		    directions[l] = _CSTAY;  // prevents others from taking the whole board
		}
	    }
	     
	    // Now for robot placement
	    
	    for (; k < size; k++) {
		returns[k] = new Robot(0, k);
		positions[k] = new OldG3Point(0, k);
	    }
	    for (; k < size * 2; k++) {
		returns[k] = new Robot(k - size, 0);
		positions[k] = new OldG3Point((k - size), 0);
	    }
	    for (; k < size * 3; k++) {
		returns[k] = new Robot(k - (size * 2), size - 1);
		positions[k] = new OldG3Point(k - (size * 2), size - 1);
	    }
	    for (; k < size * 4; k++) {
		returns[k] = new Robot(size - 1, k - (size * 3));
		positions[k] = new OldG3Point(size - 1, k - (size * 3));
	    }
	    for (; k < numRobots; k++) {
		returns[k] = new Robot(0, k);
		positions[k] = new OldG3Point(0, k);
	    }
	    
	}
	return returns;
    }
		    

    //  Moving strategy for strategy1
    //  Strategy is as follows:
    //      For 1 robot: Starting near bottom right corner, make a series of n x n squares
    //                   by moving first all left, then all right, then straight, etc.
    //      For 2 robots: Start on opposite corners, and make criss-crosses along board.
    //      For 3 robots: Same as 2 robot strategy, but more ambitious...
    //                    It will send a third robot to try and secure the bottom 1/3 
    //                    portion of the board (for x = 2, it will get the bottom 1/2)
    //      For 4 robots: Same as 3, but a fourth robot will try to get the middle 1/3.
    private char[] strategy1() throws Exception {
	if (DEBUG)
	    System.out.println("Entering strategy 1");
	char[] returns = new char[numRobots];
	switch (numRobots) {
	case 1: 
	    if (sides < 16) {
		if ((consecutive == 0) && (sides % 4 == 0)) {
		    returns[0] = turnTendency;
		    consecutive++;
		}
		else if ((consecutive == 0) && (sides % 4 != 0)) {
		    turnTendency = move90Left(turnTendency);
		    returns[0] = turnTendency;
		    consecutive++;
		}
		else if (consecutive == 3) {
		    returns[0] = turnTendency;
		    consecutive = 0;
		    sides++;
		}
		else {
		    returns[0] = turnTendency;
		    consecutive++;
		}
	    }
	    else { // sides == 16
		if (consecutive == 7) {
		    sides = 0;
		    returns[0] = turnTendency;
		    turnTendency = move90Left(turnTendency);
		    consecutive = 0;
		    numRoses++;
		}
		else {
		    if ((numRoses == 3) && (consecutive == 0))
			turnTendency = move180(turnTendency);
		    returns[0] = turnTendency;
		    consecutive++;
		}
	    }
	    break;
		
	case 4:
	    // 4 robot code goes here
	    if (positions[3].x == 0) {    // on left wall
		if (positions[3].y == ((2 * size) / 3) - 1)   // against bottom wall
		    returns[3] = _CEAST;
		else returns[3] = _CSOUTH;
	    }
	    else if (positions[3].x == size - 1) {  // on right wall
		if (numPlayers <= 2) {
		    // check to see if we're at the upper boundary for the lower part of the board
		    if (positions[3].y == (size / 3) - 1)
			returns[3] = _CWEST;
		    else returns[3] = _CNORTH;
		}
		else if (positions[3].y == 0)
		    returns[3] = _CWEST;
		else returns[3] = _CNORTH;
	    }
	    // else continue on its merry way
	    else returns[3] = directions[3];
	    
	    // no break needed here
	case 3: 
	    if (positions[2].x == 0) {    // on left wall
		if (positions[2].y == size - 1)   // against bottom wall
		    returns[2] = _CEAST;
		else returns[2] = _CSOUTH;
	    }
	    else if (positions[2].x == size - 1) {  // on right wall
		if (numPlayers != 2) {
		    // check to see if we're at the upper boundary for the lower part of the board
		    if (positions[2].y == ((2 * size) / 3) - 1)
			returns[2] = _CWEST;
		    else returns[2] = _CNORTH;
		}
		else if (positions[2].y == (size / 2) - 1)
		    returns[2] = _CWEST;
		else returns[2] = _CNORTH;
	    }
	    // else continue on its merry way
	    else returns[2] = directions[2];
	    
	    // No break needed; everything else is exactly the same as 2 robot case
	case 2:
	    // We only have one turnTendency... of course, we only need one...
	    // Note that bots 0 and 1 head in opposite directions on even 'sides', 
	    // and in the same direction on odd sides.
	    
	    // We track the building direction of the left robot.
	    // When it's reached the end of that direction, have both go up and continue

	    if (((positions[0].x == size - 1) && (! wrongway)) || 
		((positions[0].x == 0) && (wrongway))) {
		if (positions[1].y == southBoundary - 8) {
		    // flip the directions around and continue
		    System.out.println("0000000000");
		    System.out.println("executed 1");
		    turnTendency = move180(turnTendency);
		    returns[0] = turnTendency;
		    returns[1] = move180(turnTendency);
		    consecutive = 1;
		    southBoundary -= 8;
		    wrongway = !wrongway;
		}
		else {
		    returns[0] = _CNORTH;
		    returns[1] = _CNORTH;
		}
	    }
	    
	    else {
		if ((consecutive == 0) && (sides == 0)) {
		    System.out.println("Executed 2");
		    returns[0] = move180(turnTendency);
		    returns[1] = turnTendency;
		    consecutive++;
		}
		else if ((consecutive == 3) && (sides % 2 == 1)) {
		    System.out.println("Executed 3");
		    sides++;
		    returns[0] = turnTendency;
		    returns[1] = turnTendency;
		    consecutive = 0;
		    if (! wrongway)
			turnTendency = _CWEST;
		    else turnTendency = _CEAST;
		}
		else if ((consecutive == 3) && (sides % 2 == 0)) {
		    System.out.println("Executed 4");
		    returns[0] = move180(turnTendency);
		    returns[1] = turnTendency;
		    consecutive = 0;
		    if (! wrongway) {
			if (sides % 4 == 0)
			    turnTendency = move90Right(turnTendency);
			else turnTendency = move90Left(turnTendency);
		    } 
		    else {
			if (sides % 4 == 0)
			    turnTendency = move90Left(turnTendency);
			else turnTendency = move90Right(turnTendency);
		    }
		    sides++;
		}
		else if (sides % 2 == 1) {
		    System.out.println("Executed 5");
		    returns[0] = turnTendency;
		    returns[1] = turnTendency;
		    consecutive++;
		}
		else {
		    System.out.println("Executed 6");
		    returns[0] = move180(turnTendency);
		    returns[1] = turnTendency;
		    consecutive++;
		}
	    }
	}
		
	// update the positions
	for (int i = 0; i < returns.length; i++) {
	    switch(returns[i]) {
	    case _CEAST:
		if (positions[i].x != (size - 1))
		    positions[i].x++;
		directions[i] = _CEAST;
		break;
	    case _CWEST:
		if (positions[i].x != 0) 
		    positions[i].x--;
		directions[i] = _CWEST;
		break;
	    case _CNORTH:
		if (positions[i].y != 0) 
		    positions[i].y--;
		directions[i] = _CNORTH;
		break;
	    case _CSOUTH:
		if (positions[i].y != (size - 1))
		    positions[i].y++;
		directions[i] = _CSOUTH;
		break;
	    }
	}
	return returns;
    }

    // Here it is: the traditional Army strategy
    // After you've lined up the robots in a corner, march them to the other side, 
    // then move the line up (or down), repeat in the other direction, et. al.
    private char[] strategy2() throws Exception {
	if (DEBUG) {
	    System.out.println("Entering strategy 2");
	    System.out.println("size = " + size);
	    System.out.println("Current army position: " + positions[0].x);
	}
	char[] returns = directions;
	if (DEBUG)
	    System.out.println("returns array now of length " + returns.length);
	final char curDirection = directions[0];
	switch (curDirection) {
	case _CEAST:
	    if (DEBUG)
		System.out.println("IN case _CEAST");
	    // Here, keep going east until we hit the end...
	    // ...then move the whole line south/north (if we can), and move back west
	    if (positions[0].x == size - 1) {
		if (DEBUG)
		    System.out.println("positions[0].x == size - 1");
		if (vertTendency == _CSOUTH) {
		    if ((positions[numRobots - 1].y != size - 1) && (numVertical != numRobots)) {
			numVertical++;
			for (int i = 0; i < numRobots; i++) {
			    if (DEBUG)
				System.out.println("Moving everything south - not at corner");
			    returns[i] = _CSOUTH;
			    positions[i].y++;
			    directions[i] = _CSOUTH;
			}
		    }
		    else if ((positions[numRobots - 1].y != size - 1) &&
			     (numVertical == numRobots)) {
			// OK, move the robots west now
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		    else if (positions[numRobots - 1].y == size - 1) {
			// OK, we're at the corner now.
			if (DEBUG)
			    System.out.println("Changing vertTendency to NORTH");
			vertTendency = _CNORTH;
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		}
		else {  // vertTendency is north
		    if (DEBUG)
			System.out.println("vertTendency is NORTH - moving up");
		    if ((positions[0].y != 0) && (numVertical != numRobots)) {
			numVertical++;
			for (int i = 0; i < numRobots; i++) {
			    if (DEBUG)
				System.out.println("Moving up here");
			    returns[i] = _CNORTH;
			    positions[i].y--;
			    directions[i] = _CNORTH;
			}
		    }
		    else if ((positions[0].y != 0) && (numVertical == numRobots)) {
			// OK, move the robots west now
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		    else if (positions[0].y == 0) {
			// OK, we're at the corner now. ABOUT FACE!
			if (DEBUG)
			    System.out.println("Changing vertTendency to SOUTH");
			endMarch = true;
			vertTendency = _CSOUTH;
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		}
	    }
	    else {
		returns = directions;  // not at right wall; move east
		if (DEBUG)
		    System.out.println("Not at right wall");
		for (int i = 0; i < numRobots; i++) {
		    positions[i].x++;
		}
	    }
	    break;
	    
	case _CWEST: 
	    if (DEBUG)
		System.out.println("IN case _CWEST");
	    // Here, keep going west until we hit the end...
	    // ...then move the whole line south/north (if we can), and move back west
	    if (positions[0].x == 0) {
		if (DEBUG)
		    System.out.println("positions[0].x == 0");
		if (vertTendency == _CSOUTH) {
		    if ((positions[numRobots - 1].y != size - 1) && (numVertical != numRobots)) {
			numVertical++;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CSOUTH;
			    positions[i].y++;
			    directions[i] = _CSOUTH;
			}
		    }
		    else if ((positions[numRobots - 1].y != size - 1) &&
			     (numVertical == numRobots)) {
			// OK, move the robots east now
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}  
		    }
		    else if (positions[numRobots - 1].y == size - 1) {
			// OK, we're at the corner now.
			vertTendency = _CNORTH;
			if (DEBUG)
			    System.out.println("Changing vertTendency to NORTH");
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		}
		else {  // vertTendency is north
		    if ((positions[0].y != 0) && (numVertical != numRobots)) {
			numVertical++;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CNORTH;
			    positions[i].y--;
			    directions[i] = _CNORTH;
			}
		    }
		    else if ((positions[0].y != 0) && (numVertical == numRobots)) {
			// OK, move the robots east now
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		    else if (positions[0].y == 0) {
			// OK, we're at the corner now. ABOUT FACE!
			if (DEBUG)
			    System.out.println("Changing vertTendency to SOUTH");
			vertTendency = _CSOUTH;
			numVertical = 0;
			for (int i = 0; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		}
	    }
	    else {
		returns = directions;  // not at right wall; move west
		if (DEBUG)
		    System.out.println("Not at left wall");
		for (int i = 0; i < numRobots; i++) {
		    positions[i].x--;
		}
	    }
	    break;
	    
	case _CNORTH:
	    if (DEBUG)
		System.out.println("In case _CNORTH");
	    // If in a corner, move away from it...
	    // otherwise, keep going.
	    if ((positions[0].y != 0) && (numVertical != numRobots)) {
		vertTendency = _CNORTH;
		numVertical++;
		for (int i = 0; i < numRobots; i++) {
		    returns[i] = _CNORTH;
		    directions[i] = _CNORTH;
		    positions[i].y--;
		}
	    }
	    else if ((positions[0].y != 0) && (numVertical == numRobots)) {
		// OK, now move the army away
		if (positions[0].x == 0) {  // on left wall
		    numVertical = 0;
		    vertTendency = _CNORTH;
		    for (int l = 0; l < numRobots; l++) {
			returns[l] = _CEAST;
			directions[l] = _CEAST;
			positions[l].x++;
		    }
		}
		else {  // on right wall
		    vertTendency = _CNORTH;
		    numVertical = 0;
		    for (int m = 0; m < numRobots; m++) {
			returns[m] = _CWEST;
			directions[m] = _CWEST;
			positions[m].x--;
		    }
		}
	    }
	    else if (positions[0].x == 0) {
		if (DEBUG)
		    System.out.println("Suspicious statement 2 executed");
		numVertical = 0;
		vertTendency = _CSOUTH;
		for (int j = 0; j < numRobots; j++) { 
		    returns[j] = _CEAST;
		    directions[j] = _CEAST;
		    positions[j].x++;
		}
	    }
	    else {
		numVertical = 0;
		vertTendency = _CSOUTH;
		for (int k = 0; k < numRobots; k++) {
		    returns[k] = _CWEST;
		    directions[k] = _CWEST;
		    positions[k].x--;
		}
	    }
	    break;
	    
	case _CSOUTH:
	    if (DEBUG)
		System.out.println("In case _CSOUTH");
	    // If in a corner, move away from it...
	    // otherwise, keep going.
	    if ((positions[numRobots - 1].y != size - 1) && (numVertical != numRobots)) {
		vertTendency = _CSOUTH;
		numVertical++;
		for (int i = 0; i < numRobots; i++) {
		    returns[i] = _CSOUTH;
		    directions[i] = _CSOUTH;
		    positions[i].y++;
		}
	    }
	    else if ((positions[numRobots - 1].y != size - 1) && (numVertical == numRobots)) {
		// OK, now move the army away
		if (positions[0].x == 0) {  // on left wall
		    numVertical = 0;
		    vertTendency = _CSOUTH;
		    for (int l = 0; l < numRobots; l++) {
			returns[l] = _CEAST;
			directions[l] = _CEAST;
			positions[l].x++;
		    }
		}
		else {  // on right wall
		    vertTendency = _CSOUTH;
		    numVertical = 0;
		    for (int m = 0; m < numRobots; m++) {
			returns[m] = _CWEST;
			directions[m] = _CWEST;
			positions[m].x--;
		    }
		}
	    }
	    else if (positions[0].x == 0) {
		if (DEBUG)
		    System.out.println("Suspicious statement 2 executed");
		vertTendency = _CNORTH;
		numVertical = 0;
		for (int j = 0; j < numRobots; j++) { 
		    returns[j] = _CEAST;
		    directions[j] = _CEAST;
		    positions[j].x++;
		}
	    }
	    else {
		numVertical = 0;
		vertTendency = _CNORTH;
		for (int k = 0; k < numRobots; k++) {
		    returns[k] = _CWEST;
		    directions[k] = _CWEST;
		    positions[k].x--;
		}
	    }
	    break;
	default: 
	    /* Not reached! */ 
	    break;
	}
    	return returns;
    }

    // Implements strategy 3 - the split army strategy
    // The first part is simply a modified copy/paste from strategy 2, controlling the horizontal army
    // The second is amended to control the vertical army
    // Note, here, that the armies are designed to converge at the midpoint of the field.
    private char[] strategy3() throws Exception {
	if (DEBUG) 
	    System.out.println("Entering strategy 3");
	char[] returns = directions;
	
	// First, we handle the horizontal army segment
	final char curDirection = directions[0];
	switch (curDirection) {
	case _CEAST:
	    if (DEBUG)
		System.out.println("IN case _CEAST");
	    // Here, keep going east until we hit the end...
	    // ...then move the whole line south/north (if we can), and move back west
	    if (positions[0].x == size - 1) {
		if (DEBUG)
		    System.out.println("positions[0].x == size - 1");
		if (vertTendency == _CSOUTH) {
		    if ((positions[midpoint - 1].y != size - 1) && (numVertical != midpoint) && 
			(positions[midpoint - 1].y != size / 2)) {
			numVertical++;
			for (int i = 0; i < midpoint; i++) {
			    if (DEBUG)
				System.out.println("Moving everything south - not at corner");
			    returns[i] = _CSOUTH;
			    positions[i].y++;
			    directions[i] = _CSOUTH;
			}
		    }
		    else if ((positions[midpoint - 1].y != size - 1) && (numVertical == midpoint) &&
			     (positions[midpoint - 1].y != size / 2)) {
			// OK, move the robots west now
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		    else if ((positions[midpoint - 1].y == size - 1) || (positions[midpoint - 1].y == size / 2)) {
			// OK, we're at the corner now.
			if (DEBUG)
			    System.out.println("Changing vertTendency to NORTH");
			vertTendency = _CNORTH;
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		}
		else {  // vertTendency is north
		    if (DEBUG)
			System.out.println("vertTendency is NORTH - moving up");
		    if ((positions[0].y != 0) && (numVertical != midpoint) && (positions[0].y != size / 2)) {
			numVertical++;
			for (int i = 0; i < midpoint; i++) {
			    if (DEBUG)
				System.out.println("Moving up here");
			    returns[i] = _CNORTH;
			    positions[i].y--;
			    directions[i] = _CNORTH;
			}
		    }
		    else if ((positions[0].y != 0) && (numVertical == midpoint) && (positions[0].y != size / 2)) {
			// OK, move the robots west now
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		    else if ((positions[0].y == 0) || (positions[0].y == size / 2)) {
			// OK, we're at the corner now. ABOUT FACE!
			if (DEBUG)
			    System.out.println("Changing vertTendency to SOUTH");
			endMarch = true;
			vertTendency = _CSOUTH;
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		}
	    }
	    else {
		for (int i = 0; i < midpoint; i++) {
		    returns[i] = directions[i];  // not at right wall; move east
		    positions[i].x++;
		}
		if (DEBUG)
		    System.out.println("Not at right wall");
	    }
	    break;
	    
	    
	case _CWEST: 
	    if (DEBUG)
		System.out.println("IN case _CWEST");
	    // Here, keep going west until we hit the end...
	    // ...then move the whole line south/north (if we can), and move back west
	    if (positions[0].x == 0) {
		if (DEBUG)
		    System.out.println("positions[0].x == 0");
		if (vertTendency == _CSOUTH) {
		    if ((positions[midpoint - 1].y != size - 1) && (numVertical != midpoint) && 
			(positions[midpoint - 1].y != size / 2)) {
			numVertical++;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CSOUTH;
			    positions[i].y++;
			    directions[i] = _CSOUTH;
			}
		    }
		    else if ((positions[midpoint].y != size - 1) && (numVertical == midpoint) && 
			     (positions[midpoint].y != size / 2)) {
			// OK, move the robots east now
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}  
		    }
		    else if ((positions[midpoint].y == size - 1) || (positions[midpoint].y == size / 2)) {
			// OK, we're at the corner now.
			vertTendency = _CNORTH;
			if (DEBUG)
			    System.out.println("Changing vertTendency to NORTH");
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		}
		else {  // vertTendency is north
		    if ((positions[0].y != 0) && (numVertical != midpoint) && (positions[0].y != size / 2)) {
			numVertical++;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CNORTH;
			    positions[i].y--;
			    directions[i] = _CNORTH;
			}
		    }
		    else if ((positions[0].y != 0) && (numVertical == midpoint) && (positions[0].y != size / 2)) {
			// OK, move the robots east now
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		    else if ((positions[0].y == 0) || (positions[0].y == size / 2)) {
			// OK, we're at the corner now. ABOUT FACE!
			if (DEBUG)
			    System.out.println("Changing vertTendency to SOUTH");
			vertTendency = _CSOUTH;
			numVertical = 0;
			for (int i = 0; i < midpoint; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		}
	    }
	    else {
		for (int i = 0; i < midpoint; i++) {
		    returns[i] = directions[i];  // not at left wall; move west
		    positions[i].x--;
		}

		if (DEBUG)
		    System.out.println("Not at left wall");
	    }
	    break;
	    
	case _CNORTH:
	    if (DEBUG)
		System.out.println("In case _CNORTH");
	    // If in a corner, move away from it...
	    // otherwise, keep going.
	    if ((positions[0].y != 0) && (numVertical != midpoint) && (positions[0].y != size / 2)) {
		vertTendency = _CNORTH;
		numVertical++;
		for (int i = 0; i < midpoint; i++) {
		    returns[i] = _CNORTH;
		    directions[i] = _CNORTH;
		    positions[i].y--;
		}
	    }
	    else if ((positions[0].y != 0) && (numVertical == midpoint) && (positions[0].y != size / 2)) {
		// OK, now move the army away
		if (positions[0].x == 0) {  // on left wall
		    numVertical = 0;
		    vertTendency = _CNORTH;
		    for (int l = 0; l < midpoint; l++) {
			returns[l] = _CEAST;
			directions[l] = _CEAST;
			positions[l].x++;
		    }
		}
		else {  // on right wall
		    vertTendency = _CNORTH;
		    numVertical = 0;
		    for (int m = 0; m < midpoint; m++) {
			returns[m] = _CWEST;
			directions[m] = _CWEST;
			positions[m].x--;
		    }
		}
	    }
	    else if (positions[0].x == 0) {
		if (DEBUG)
		    System.out.println("Suspicious statement 2 executed");
		numVertical = 0;
		vertTendency = _CSOUTH;
		for (int j = 0; j < midpoint; j++) { 
		    returns[j] = _CEAST;
		    directions[j] = _CEAST;
		    positions[j].x++;
		}
	    }
	    else {
		numVertical = 0;
		vertTendency = _CSOUTH;
		for (int k = 0; k < midpoint; k++) {
		    returns[k] = _CWEST;
		    directions[k] = _CWEST;
		    positions[k].x--;
		}
	    }
	    break;
	    
	case _CSOUTH:
	    if (DEBUG)
		System.out.println("In case _CSOUTH");
	    // If in a corner, move away from it...
	    // otherwise, keep going.
	    if ((positions[midpoint - 1].y != size - 1) && (numVertical != midpoint) && 
		(positions[midpoint - 1].y != size / 2)) {
		vertTendency = _CSOUTH;
		numVertical++;
		for (int i = 0; i < midpoint; i++) {
		    returns[i] = _CSOUTH;
		    directions[i] = _CSOUTH;
		    positions[i].y++;
		}
	    }
	    else if ((positions[midpoint - 1].y != size - 1) && (numVertical == midpoint) &&
		     (positions[midpoint - 1].y != size / 2)) {
		// OK, now move the army away
		if (positions[0].x == 0) {  // on left wall
		    numVertical = 0;
		    vertTendency = _CSOUTH;
		    for (int l = 0; l < midpoint; l++) {
			returns[l] = _CEAST;
			directions[l] = _CEAST;
			positions[l].x++;
		    }
		}
		else {  // on right wall
		    vertTendency = _CSOUTH;
		    numVertical = 0;
		    for (int m = 0; m < midpoint; m++) {
			returns[m] = _CWEST;
			directions[m] = _CWEST;
			positions[m].x--;
		    }
		}
	    }
	    else if (positions[0].x == 0) {
		if (DEBUG)
		    System.out.println("Suspicious statement 2 executed");
		vertTendency = _CNORTH;
		numVertical = 0;
		for (int j = 0; j < midpoint; j++) { 
		    returns[j] = _CEAST;
		    directions[j] = _CEAST;
		    positions[j].x++;
		}
	    }
	    else {
		numVertical = 0;
		vertTendency = _CNORTH;
		for (int k = 0; k < midpoint; k++) {
		    returns[k] = _CWEST;
		    directions[k] = _CWEST;
		    positions[k].x--;
		}
	    }
	    break;

	default: 
	    /* Not reached! */ 
	    break;
	}

	// Now we handle the second half of the army
	// Note that this half of the army now marches horizontally, whereas 
	// in release 0.3 it marched vertically
	final char curDirection2 = directions[midpoint];
	switch (curDirection2) {
	case _CEAST:
	    if (DEBUG)
		System.out.println("IN case _CEAST");
	    // Here, keep going east until we hit the end...
	    // ...then move the whole line south/north (if we can), and move back west
	    if (positions[midpoint].x == size - 1) {
		if (DEBUG)
		    System.out.println("positions[0].x == size - 1");
		if (vertTendency2 == _CSOUTH) {
		    if ((positions[numRobots - 1].y != size - 1) && (numVertical2 != (numRobots - midpoint)) && 
			(positions[numRobots - 1].y != size / 2)) {
			numVertical2++;
			for (int i = midpoint; i < numRobots; i++) {
			    if (DEBUG)
				System.out.println("Moving everything south - not at corner");
			    returns[i] = _CSOUTH;
			    positions[i].y++;
			    directions[i] = _CSOUTH;
			}
		    }
		    else if ((positions[numRobots - 1].y != size - 1) && (numVertical2 == (numRobots - midpoint))
			     && (positions[numRobots - 1].y != size / 2)) {
			// OK, move the robots west now
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		    else if ((positions[numRobots - 1].y == size - 1) || (positions[numRobots - 1].y == size / 2)) {
			// OK, we're at the corner now.
			if (DEBUG)
			    System.out.println("Changing vertTendency to NORTH");
			vertTendency2 = _CNORTH;
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		}
		else {  // vertTendency is north
		    if (DEBUG)
			System.out.println("vertTendency2 is NORTH - moving up");
		    if ((positions[midpoint].y != 0) && (numVertical2 != (numRobots - midpoint)) 
			&& (positions[midpoint].y != size / 2)) {
			numVertical2++;
			for (int i = midpoint; i < numRobots; i++) {
			    if (DEBUG)
				System.out.println("Moving up here");
			    returns[i] = _CNORTH;
			    positions[i].y--;
			    directions[i] = _CNORTH;
			}
		    }
		    else if ((positions[midpoint].y != 0) && (numVertical2 == (numRobots - midpoint)) && 
			     (positions[midpoint].y != size / 2)) {
			// OK, move the robots west now
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		    else if ((positions[midpoint].y == 0) || (positions[midpoint].y == size / 2)) {
			// OK, we're at the corner now. ABOUT FACE!
			if (DEBUG)
			    System.out.println("Changing vertTendency to SOUTH");
			endMarch = true;
			vertTendency2 = _CSOUTH;
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CWEST;
			    positions[i].x--;
			    directions[i] = _CWEST;
			}
		    }
		}
	    }
	    else {
		for (int i = midpoint; i < numRobots; i++) {
		    returns[i] = directions[i];  // not at right wall; move east
		    positions[i].x++;
		}
		if (DEBUG)
		    System.out.println("Not at right wall");
	    }
	    break;
	    
	    
	case _CWEST: 
	    if (DEBUG)
		System.out.println("IN case _CWEST");
	    // Here, keep going west until we hit the end...
	    // ...then move the whole line south/north (if we can), and move back west
	    if (positions[midpoint].x == 0) {
		if (DEBUG)
		    System.out.println("positions[0].x == 0");
		if (vertTendency2 == _CSOUTH) {
		    if ((positions[numRobots - 1].y != size - 1) && (numVertical2 != (numRobots - midpoint)) && 
			(positions[numRobots - 1].y != size / 2)) {
			numVertical2++;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CSOUTH;
			    positions[i].y++;
			    directions[i] = _CSOUTH;
			}
		    }
		    else if ((positions[midpoint].y != size - 1) && (numVertical2 == midpoint) && 
			     (positions[midpoint].y != size / 2)) {
			// OK, move the robots east now
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}  
		    }
		    else if ((positions[midpoint].y == size - 1) || (positions[midpoint].y == size / 2)) {
			// OK, we're at the corner now.
			vertTendency2 = _CNORTH;
			if (DEBUG)
			    System.out.println("Changing vertTendency2 to NORTH");
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		}
		else {  // vertTendency2 is north
		    if ((positions[midpoint].y != 0) && (numVertical2 != midpoint) && (positions[midpoint].y != size / 2)) {
			numVertical2++;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CNORTH;
			    positions[i].y--;
			    directions[i] = _CNORTH;
			}
		    }
		    else if ((positions[midpoint].y != 0) && (numVertical == (numRobots - midpoint)) && 
			     (positions[midpoint].y != size / 2)) {
			// OK, move the robots east now
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		    else if ((positions[midpoint].y == 0) || (positions[midpoint].y == size / 2)) {
			// OK, we're at the corner now. ABOUT FACE!
			if (DEBUG)
			    System.out.println("Changing vertTendency to SOUTH");
			vertTendency2 = _CSOUTH;
			numVertical2 = 0;
			for (int i = midpoint; i < numRobots; i++) {
			    returns[i] = _CEAST;
			    positions[i].x++;
			    directions[i] = _CEAST;
			}
		    }
		}
	    }
	    else {
		for (int i = midpoint; i < numRobots; i++) {
		    returns[i] = directions[i];  // not at left wall; move west
		    positions[i].x--;
		}

		if (DEBUG)
		    System.out.println("Not at left wall");
	    }
	    break;
	    
	case _CNORTH:
	    if (DEBUG)
		System.out.println("In case _CNORTH");
	    // If in a corner, move away from it...
	    // otherwise, keep going.
	    if ((positions[midpoint].y != 0) && (numVertical2 != (numRobots - midpoint)) && 
		(positions[midpoint].y != size / 2)) {
		vertTendency2 = _CNORTH;
		numVertical2++;
		for (int i = midpoint; i < numRobots; i++) {
		    returns[i] = _CNORTH;
		    directions[i] = _CNORTH;
		    positions[i].y--;
		}
	    }
	    else if ((positions[midpoint].y != 0) && (numVertical2 == (numRobots - midpoint)) 
		     && (positions[midpoint].y != size / 2)) {
		// OK, now move the army away
		if (positions[midpoint].x == 0) {  // on left wall
		    numVertical2 = 0;
		    vertTendency2 = _CNORTH;
		    for (int l = midpoint; l < numRobots; l++) {
			returns[l] = _CEAST;
			directions[l] = _CEAST;
			positions[l].x++;
		    }
		}
		else {  // on right wall
		    vertTendency2 = _CNORTH;
		    numVertical2 = 0;
		    for (int m = midpoint; m < numRobots; m++) {
			returns[m] = _CWEST;
			directions[m] = _CWEST;
			positions[m].x--;
		    }
		}
	    }
	    else if (positions[midpoint].x == 0) {
		if (DEBUG)
		    System.out.println("Suspicious statement 2 executed");
		numVertical2 = 0;
		vertTendency2 = _CSOUTH;
		for (int j = midpoint; j < numRobots; j++) { 
		    returns[j] = _CEAST;
		    directions[j] = _CEAST;
		    positions[j].x++;
		}
	    }
	    else {
		numVertical2 = 0;
		vertTendency2 = _CSOUTH;
		for (int k = midpoint; k < numRobots; k++) {
		    returns[k] = _CWEST;
		    directions[k] = _CWEST;
		    positions[k].x--;
		}
	    }
	    break;
	    
	case _CSOUTH:
	    if (DEBUG)
		System.out.println("In case _CSOUTH");
	    // If in a corner, move away from it...
	    // otherwise, keep going.
	    if ((positions[numRobots - 1].y != size - 1) && (numVertical2 != (numRobots - midpoint)) && 
		(positions[numRobots - 1].y != size / 2)) {
		vertTendency2 = _CSOUTH;
		numVertical2++;
		for (int i = midpoint; i < numRobots; i++) {
		    returns[i] = _CSOUTH;
		    directions[i] = _CSOUTH;
		    positions[i].y++;
		}
	    }
	    else if ((positions[numRobots - 1].y != size - 1) && (numVertical2 == (numRobots - midpoint)) &&
		     (positions[numRobots - 1].y != size / 2)) {
		// OK, now move the army away
		if (positions[midpoint].x == 0) {  // on left wall
		    numVertical2 = 0;
		    vertTendency2 = _CSOUTH;
		    for (int l = midpoint; l < numRobots; l++) {
			returns[l] = _CEAST;
			directions[l] = _CEAST;
			positions[l].x++;
		    }
		}
		else {  // on right wall
		    vertTendency2 = _CSOUTH;
		    numVertical2 = 0;
		    for (int m = midpoint; m < numRobots; m++) {
			returns[m] = _CWEST;
			directions[m] = _CWEST;
			positions[m].x--;
		    }
		}
	    }
	    else if (positions[midpoint].x == 0) {
		if (DEBUG)
		    System.out.println("Suspicious statement 2 executed");
		vertTendency2 = _CNORTH;
		numVertical2 = 0;
		for (int j = midpoint; j < numRobots; j++) { 
		    returns[j] = _CEAST;
		    directions[j] = _CEAST;
		    positions[j].x++;
		}
	    }
	    else {
		numVertical2 = 0;
		vertTendency2 = _CNORTH;
		for (int k = midpoint; k < numRobots; k++) {
		    returns[k] = _CWEST;
		    directions[k] = _CWEST;
		    positions[k].x--;
		}
	    }
	    break;

	default: 
	    /* Not reached! */ 
	    break;
	}
    	return returns;
    }

    // Strategy 4: Here, one of five scenarios occur:
    // 1) For numRobots == size: just walk back and forth across the board
    // 2) For size < numRobots <= 2 * size: walk in opposite directions
    // 3) For size * 2 < numRobots <= 4 * size: same direction, whole rectangle
    // 4) For numRobots > 4 * size, and numPlayers <= 3: stay put
    // 5) For numRobots > 4 * size, and numPlayers > 3: walk to opposite wall and back
    private char[] strategy4() {
	char[] returns = new char[numRobots];
	// If we are staying put, then stay put.
	if (directions[0] == _CSTAY) 
	    returns = directions;
	// otherwise, if we are gonna get going, get going!
	else if (numRobots == size) {   // just go back and forth...
	    if (positions[0].x == 0) {
		for (int i = 0; i < numRobots; i++) {
		    positions[i].x++;
		    directions[i] = _CEAST;
		    returns[i] = _CEAST;
		}
	    }
	    else if (positions[0].x == (size - 1)) {
		for (int j = 0; j < numRobots; j++) {
		    positions[j].x--;
		    directions[j] = _CWEST;
		    returns[j] = _CWEST;
		}
	    }
	    else { 
		returns = directions;
		for (int i = 0; i < numRobots; i++) {
		    if (directions[0] == _CEAST)
			positions[i].x++;
		    else positions[i].x--;
		}
	    }
	}
	else if ((numRobots > size) && (numRobots <= size * 2)) {
	    // If we have hit a wall, turn around
	    // otherwise, keep going
	    if (((positions[0].x == size - 1) && (directions[0] == _CEAST))
		|| ((positions[0].x == 0) && (directions[0] == _CWEST))) { 
		for (int i = 0; i < numRobots; i++) {
		    returns[i] = move180(directions[i]);
		    directions[i] = move180(directions[i]);
		    if (directions[i] == _CEAST)
			positions[i].x++;
		    else positions[i].x--;
		}
	    }
	    else {
		returns = directions;
		for (int i = 0; i < numRobots; i++) {
		    if (directions[i] == _CEAST)
			positions[i].x++;
		    else positions[i].x--;
		}
	    }
	}
	else if ((numRobots > size * 2) && (numRobots <= size * 4)) {
	    // Remember: up to size on left of rectangle,
	    // up to size * 2 on right of rectangle,
	    // and the rest split up on either end
	    if (((positions[size].x == size - 1) && (directions[size] == _CEAST)) || 
	        ((positions[0].x == size - 1) || (directions[0] == _CEAST))){
		// On right wall; turn around!
		for (int i = 0; i < numRobots; i++) {
		    returns[i] = move180(directions[i]);
		    directions[i] = move180(directions[i]);
		    positions[i].x--;
		}
	    }
	    else if (((positions[0].x == 0) && (directions[0] == _CWEST)) || 
		     ((positions[size].x == size - 1) && (directions[size] == _CWEST))){
		// On left wall; turn around!
		for (int i = 0; i < numRobots; i++) {
		    returns[i] = move180(directions[i]);
		    directions[i] = move180(directions[i]);
		    positions[i].x++;
		}
	    }
	    else {
		returns = directions;
		for (int i = 0; i < numRobots; i++) {
		    if (directions[i] == _CEAST)
			positions[i].x++;
		    else positions[i].x--;
		}
	    }
	}
	else {  // numRobots >= size * 4
	    // Note that each robot has the same distance to travel to get to the 
	    // other end of the board!
	    // So, test one robot, you've tested them all!
	    if (((positions[0].x == size - 1) && (directions[0] == _CEAST)) ||
		(positions[0].x == 0) && (directions[0] == _CWEST)) {
		for (int i = 0; i < numRobots; i++) {
		    if (directions[i] == _CSTAY) {
			returns[i] = _CSTAY;
		    }
		    else {
			returns[i] = move180(directions[i]);
			directions[i] = move180(directions[i]);
			switch(directions[i]) {
			case _CNORTH: 
			    positions[i].y--;
			    break;
			case _CSOUTH:
			    positions[i].y++;
			    break;
			case _CEAST:
			    positions[i].x++;
			    break;
			case _CWEST:
			    positions[i].x--;
			    break;
			}
		    }
		}
	    }
	    else {
		returns = directions;
		for (int i = 0; i < numRobots; i++) {
		    switch(directions[i]) {
			case _CNORTH: 
			    positions[i].y--;
			    break;
			case _CSOUTH:
			    positions[i].y++;
			    break;
			case _CEAST:
			    positions[i].x++;
			    break;
			case _CWEST:
			    positions[i].x--;
			    break;
		    default:
			// do nothing
			break;
		    }
		}
	    }   
	}
	return returns;
    }   
	    

			

	     
	    
	    
    public char[] move() throws Exception {
	numRobots = _rect.numRobots();
	if (numRobots <= LOW) {
	    return strategy1();
	}
	else if (numRobots < size) { 
	    if ((numPlayers > 3) || (numRobots < ROBTHRESH) || 
		 (robotRatio >= UPPERTHRESH) || (numRobots > size)) 
		return strategy2();
	    else return strategy3();
	}
	else return strategy4();
    }

    public String name() throws Exception {
        return name;
    }

    public Color color() throws Exception {
        return _CCOLOR;
    }

    public boolean interactive() throws Exception {
        return false;
    }        
}
