package Rectangles;

import ui.*;
import java.util.*;
import java.io.*;
import java.awt.Color;

public final class OldGroup5PlayerU implements IFCPlayer {
  
  Rectangles rectangles;
  static final String name = "Robots' Union";
  static final Color color = new Color(0.0f, 0.6f, 1.0f);  
  static Random random;

  int num_robots;
  int num_unleashed;
  int board_size;
  int me;
  int my_last_score;
  int celebrating;

  OldGroup5Robot robots[];

  int num_segments;

  public Robot[] register(Rectangles rectangles) throws Exception {
    this.rectangles = rectangles;
    my_last_score = 0;
    celebrating = 0;
    random = new Random();
    
    num_robots = rectangles.numRobots();
    num_unleashed = 0;
    board_size = rectangles.size();
    me = rectangles.indexOf(this);
    
    num_segments = 3;
    if (board_size > 32) num_segments = 5;

    robots = new OldGroup5Robot[num_robots];
    
    for (int i = 0; i < num_robots; i++) {
      OldGroup5Robot robot;

      int x = random.nextInt(board_size);
      int y = random.nextInt(board_size);
      
      x = nearestStationCoord (x);
      y = nearestStationCoord (y);

      robot = new OldGroup5Robot (x, y, rectangles, me, random);
      robots[i] = robot;
    }

    double total_robots = num_robots * rectangles.numPlayers();
    double square_size = board_size * board_size;

    if (square_size / total_robots < 40.0 ||
        board_size <= 10 || 
        board_size <= num_robots) {
      for (int i = 0; i < num_robots; i++)
        robots[i].unleashed = true;
      num_unleashed = num_robots;
      celebrating = -10;
    }
    
    return robots;
  }
  
  public char[] move() throws Exception {
    if (celebrating < 0) celebrating = -celebrating;

    if (celebrating > 0) {
      celebrating--;
    }

    char move[] = new char[num_robots];
    
    double seg_per = brokenSegmentRatio();    
    int num_to_unleash = numToUnleash (seg_per);
    
    // System.err.println (seg_per * 100 + "%, #" + num_to_unleash);

    while (num_unleashed < num_to_unleash) {
      OldGroup5Robot robot = robots[random.nextInt (num_robots)];
      if (robot.unleashed == false) {
        robot.unleashed = true;
        num_unleashed++;
        celebrating = 1;
      }
    }

    for (int i = 0; i < num_robots; i++)
      move[i] = robots[i].move(this);   
    
    return move;
  }

  public Vertex whereNext (Vertex where_now, OldGroup5Robot robot) throws Exception {
    if ( ! isStation (where_now)) {
      return nearestStation (where_now);
    }
      
    Vertex stations[] = nearbyStations (where_now);
    return bestStation (where_now, stations, robot);
  }

  private boolean isStation (Vertex v) throws Exception {
    if ( ! isStationCoord (v.xpos())) 
      return false;

    if ( ! isStationCoord (v.ypos())) 
      return false;
    
    return true;
  }

  private boolean isStationCoord (int c) throws Exception {
    for (int i = 0; i <= num_segments; i++) {
      if (c == i * (board_size - 1) / num_segments)
        return true;
    }
    return false;     
  }
  
  private Vertex nearestStation (Vertex v) throws Exception {
    int x = nearestStationCoord(v.xpos());
    int y = nearestStationCoord(v.ypos());
    return new Vertex (x, y);
  }

  private int nearestStationCoord (int c) throws Exception {
    if (isStationCoord (c)) return c;

    int before = 0;
    int after = board_size - 1;
    for (int i = 0; i <= num_segments; i++) {
      if (c > i * (board_size - 1) / num_segments) {
        before = i * (board_size - 1) / num_segments;
      } else if (c < i * (board_size - 1) / num_segments) {
        after = i * (board_size - 1) / num_segments;
        break;
      } else {
        throw new Exception ("What?");
      }
    }

    if (c - before > after - c) {
      return before;
    } else {
      return after;
    }
  }

  private Vertex[] nearbyStations (Vertex v) throws Exception {

    if ( ! isStation(v)) {
      throw new Exception ("Not a station");
    }

    Vector vec = new Vector();

    int x = v.xpos();
    int y = v.ypos();

    int last_x;
    int last_y;
    int next_x;
    int next_y;

    if (x == 0) {
      last_x = -1;
    } else {
      for (last_x = x - 1; ! isStationCoord (last_x); last_x--);
    }

    if (y == 0) {
      last_y = -1;
    } else {
      for (last_y = y - 1; ! isStationCoord (last_y); last_y--);
    }

    if (x == board_size - 1) {
      next_x = -1;
    } else {
      for (next_x = x + 1; ! isStationCoord (next_x); next_x++);
    }

    if (y == board_size - 1) {
      next_y = -1;
    } else {
      for (next_y = y + 1; ! isStationCoord (next_y); next_y++);
    }

    if (last_x > -1) vec.add (new Vertex (last_x, y));
    if (next_x > -1) vec.add (new Vertex (next_x, y));
    if (last_y > -1) vec.add (new Vertex (x, last_y));
    if (next_y > -1) vec.add (new Vertex (x, next_y));
    
    Object o_array[] = vec.toArray();
    Vertex vertices[] = new Vertex[o_array.length];
    for (int i = 0; i < o_array.length; i++) {
      vertices[i] = (Vertex) o_array[i];
    }

    return vertices;
  }

  private Vertex bestStation (Vertex where_now, Vertex stations[], OldGroup5Robot robot) 
    throws Exception 
  {
    // System.err.println ("Where next from " + where_now + "?");

    double bestScore;
    Vector winners;
    Vector current = new Vector();

    for (int i = 0; i < stations.length; i++)
      current.add (stations[i]);

    for (int distance = 1; ; distance++) {
      winners = null;
      bestScore = -65000.0;

      if (distance > num_segments + 2) {
        throw new Exception ("Clockwise led to a tie???");
      } else if (distance > num_segments + 1) {
        // System.err.println ("Scores for clockwise");
      } else {
        // System.err.println ("Scores for distance " + distance);
      }

      for (int i = 0; i < current.size() ; i++) {
        Vertex station = (Vertex) current.elementAt(i);
        char direction = getDirection (where_now, station);
        double score;

        if (distance > num_segments + 1) {
          score = clockwise (where_now, direction);
        } else {
          score = score (where_now, direction, distance, robot.inertia);
        }
        if (winners == null || score > bestScore) {
          bestScore = score;
          winners = new Vector();
          winners.add (stations[i]);
        } else if (score == bestScore) {
          winners.add (stations[i]);
        }
        // System.err.println (stations[i] + ": " + score);
      }               

      if (winners == null) throw new Exception ("Huh?");
      if (winners.size() == 0) throw new Exception ("What?");
      if (winners.size() == 1) break;

      // System.err.println ("Tie! Time for a runoff.");

      current = winners;
    }

    //System.err.println ("Best score: " + bestScore);

    Vertex v = (Vertex) winners.elementAt(0);
    // System.err.println ("We're going with " + v);
    // System.err.println ();

    return v;
  }

  private char getDirection (Vertex here, Vertex there) throws Exception {
    int hereX = here.xpos();
    int hereY = here.ypos();
    int thereX = there.xpos();
    int thereY = there.ypos();

    if (hereX < thereX) return _CEAST;
    if (hereX > thereX) return _CWEST;
    if (hereY < thereY) return _CSOUTH;
    if (hereY > thereY) return _CNORTH;

    throw new Exception ("What?");
  }

  private double score (Vertex here, char direction, int distance, char inertia)
    throws Exception
  {
    int points[] = points (here, direction, distance);

    boolean kicker = false;
    boolean rivals = false;
    boolean whites = false;
    boolean filled = false;
    boolean others = false;

    for (int i = 1; i < points.length; i++) {
      if (points[i] == -10 /* crowded */) {
        others = true;
      } else if (points[i] == _CFILLED) {
        filled = true;
      } else if (points[i] == me) {
        if (i == points.length - 1) 
          kicker = true;
      } else if (points[i] == _CEMPTY) {
        whites = true;
      } else if (points[i] == _CCONTESTED) {
        whites = true;
      } else {
        rivals = true;
      }
    }

    double score = 0.0;

    if (kicker) score +=  1.0;
    if (rivals) score +=  2.0;
    if (inertia == direction) score += 4.0;
    if (whites) score +=  8.0;
    if (others) score -= 16.0;
    if (filled) score -= 32.0;

    return score;
  }

  private double clockwise (Vertex here, char direction) throws Exception {
    int x = here.xpos();
    int y = here.ypos();

    int midpoint = board_size / 2;

    if (x < midpoint && y < midpoint && direction == _CEAST) return 3.0;
    if (x < midpoint && y < midpoint && direction == _CNORTH) return 2.0;
    if (x < midpoint && y < midpoint && direction == _CWEST) return 1.0;
    if (x < midpoint && y < midpoint && direction == _CSOUTH) return 0.0;

    if (x >= midpoint && y < midpoint && direction == _CSOUTH) return 3.0;
    if (x >= midpoint && y < midpoint && direction == _CEAST) return 2.0;
    if (x >= midpoint && y < midpoint && direction == _CNORTH) return 1.0;
    if (x >= midpoint && y < midpoint && direction == _CWEST) return 0.0;

    if (x >= midpoint && y >= midpoint && direction == _CWEST) return 3.0;
    if (x >= midpoint && y >= midpoint && direction == _CSOUTH) return 2.0;
    if (x >= midpoint && y >= midpoint && direction == _CEAST) return 1.0;
    if (x >= midpoint && y >= midpoint && direction == _CNORTH) return 0.0;

    if (x < midpoint && y >= midpoint && direction == _CNORTH) return 3.0;
    if (x < midpoint && y >= midpoint && direction == _CWEST) return 2.0;
    if (x < midpoint && y >= midpoint && direction == _CSOUTH) return 1.0;
    if (x < midpoint && y >= midpoint && direction == _CEAST) return 0.0;

    throw new Exception ("Guh?");
  }

  private int[] points (Vertex here, char direction, int distance)
    throws Exception 
  {
    int x = here.xpos();
    int y = here.ypos();
    int board[][] = rectangles.colors();
    Vector v = new Vector();
    int stations_seen = 0;

    while (true) {
      if (isStation (new Vertex (x, y))) stations_seen++;

      if (crowded (new Vertex (x, y))) {
        v.add (new Integer (-10));
      } else {
        v.add (new Integer(board[x][y]));
      }

      if (stations_seen > distance) break;

      if (direction == _CEAST) x++;
      if (direction == _CWEST) x--;
      if (direction == _CSOUTH) y++;
      if (direction == _CNORTH) y--;

      if (x < 0) break;
      if (x > board_size - 1) break;
      if (y < 0) break;
      if (y > board_size - 1) break;
    }

    Object o_array[] = v.toArray();
    int points[] = new int[o_array.length];
    for (int i = 0; i < o_array.length; i++) {
      points[i] = ((Integer)(o_array[i])).intValue();
    }

    return points;
  }
  
  private int getScore () throws Exception {
    MoveResult history[] = rectangles.history();
    if (history.length == 0) return 0;
    MoveResult lastResult = history[history.length - 1];
    double scores[] = lastResult.scores();
    return (int) scores[me];
  }

  public String name() throws Exception {
    return name;
  }
  
  public Color color() throws Exception {
    Color c = color;

    if (celebrating > 0) {
      float green = 0.30f + 0.35f * random.nextFloat();
      c = new Color(0.1f + 0.3f * random.nextFloat(),
                    green,
                    green + 0.1f + (0.9f - green) * random.nextFloat());
    }

    return c;
  }
  
  public boolean interactive() throws Exception {
    return false;
  }

  private boolean crowded (Vertex v) throws Exception {
    Robot all[][] = rectangles.allRobots();
    Robot mine[] = all[me];

    for (int i = 0; i < mine.length; i++) {
      Robot robot = mine[i];
      if (robot.xpos() == v.xpos() && robot.ypos() == v.ypos()) return true;
    }

    return false;
  }

  private double brokenSegmentRatio () throws Exception {
    double good = 0.0;
    double bad = 0.0;

    for (int y = 1; y < num_segments; y++)
      for (int x = 0; x < num_segments; x++)
        if (noBlack (station(x), station(y), true))
          good += 1.0;
        else
          bad += 1.0;

    for (int x = 1; x < num_segments; x++)
      for (int y = 0; y < num_segments; y++)
        if (noBlack (station(x), station(y), false))
          good += 1.0;
        else
          bad += 1.0;
    
    //System.err.print ("good: " + good + ", bad: " + bad + ", ");
    return bad / (good + bad);
  }

  private int station (int c) throws Exception {
    int stations_seen = -1;

    for (int i = 0; i < board_size; i++) {
      if (isStationCoord(i))
        stations_seen++;

      if (stations_seen == c)
        return i;
    }
    
    throw new Exception ("station overflow");
  }

  private boolean noBlack (int x, int y, boolean right) throws Exception {    
    int board[][] = rectangles.colors();

    if (board[x][y] == _CFILLED) return false;

    if (right) x++; else y++;

    while (true) {
      if (board[x][y] == _CFILLED) return false;
      if (right) {
        if (isStationCoord (x++)) return true;
      } else {
        if (isStationCoord (y++)) return true;
      }
    }
  }

  private int numToUnleash (double seg_per) throws Exception {
    if (seg_per < 0.25)
      return 0;
      
    if (seg_per >= 0.75)
      return num_robots;

    if (num_robots == 1) return 0;    
    if (num_robots == 2) return 1;

    double stretched = (seg_per - 0.25) * 2.0;

    if (stretched < 0.0) throw new Exception ("underflow");
    if (stretched > 1.0) throw new Exception ("overflow");

    return 1 + (int)(stretched * num_robots);
  }  
}
