//***********************************************************
//*
//* File:           Group6Player3.java
//* Author:         EZ, MC, MP
//* Contact:        matias@cs
//* Update:         10.27.2002
//*
//* Description: recognizes being chased.
//*
//*
//***********************************************************

package Rectangles;

import ui.*;

import java.util.*;
import java.io.*;
import java.awt.Color;

public final class OldGroup6Player3 implements IFCPlayer {

    private Rectangles engine;
    private int[][] colors;
    private static Random random;
    private static final String NAME = "Group6Player3";
    private static final Color COLOR = new Color(1.0f, 0.5f, 1.0f).darker();
    private static final double BUILDER_RATIO = .8;

    private int numRobots, boardSize;
    private LogicRobot[] robots;
    private static final boolean DEBUG = true;

    static {
	try {
	    random = java.security.SecureRandom.getInstance("SHA1PRNG");
	} catch (Exception e) {
	    System.err.println("eep! Couldn't get secure RNG: " + e);
	    random = new Random();
	}
    }

    public Robot[] register(Rectangles engine) throws Exception {
        this.engine = engine;

        numRobots = this.engine.numRobots();
        int numBuilders = Math.max(2, (int) (numRobots * BUILDER_RATIO));
        boardSize = this.engine.size();
        robots = new LogicRobot[numRobots];

        // create robots with initial positions

        int squareSize = 5;
        if (numRobots > 5)
            squareSize = 3;
        else
            squareSize = 4;

        int initialX = 0;
        int y = squareSize * 2 - squareSize/2;
        int delta = squareSize*3 - (squareSize/3);
        int heading = DiagonalSquaresLogic.DIAG_DOWN;

        boolean reverse = random.nextBoolean();
        if (reverse) {
            initialX = boardSize-1;
            y = boardSize - squareSize*2 + squareSize/2;
            delta = -delta;
            heading = DiagonalSquaresLogic.DIAG_UP;
        }

        int x = initialX;
        for (int i = 0; i < numRobots; i++) {

            if (i < numBuilders) {
                if (i > numBuilders / 2 || boardSize - Math.abs(x - initialX) < squareSize * 2) {
                    debug("moving position");
                    x = initialX;
                    y += delta;
                }

                robots[i] = new LogicRobot(x, y);
                robots[i].setLogic(new DiagonalSquaresLogic(robots[i], squareSize, heading));
                x += delta;

            } else {
		// deconstructive robots get placed randomly
                robots[i] = new LogicRobot(random.nextInt(boardSize),
					   random.nextInt(boardSize));
                robots[i].setLogic(new DisruptLogic(robots[i]));
            }

        }

        // do a sanity check to make sure robots are inside bounds
        for (int i = 0; i < robots.length; i++) {
            if (robots[i].xpos() > boardSize - 1 || robots[i].xpos() < 0) {
                robots[i].setXpos(0);
                debug("reset x position of robot " + i);
            }

            if (robots[i].ypos() > boardSize - 1 || robots[i].ypos() < 0) {
                robots[i].setYpos(0);
                debug("reset y position of robot " + i);
            }

        }

        return robots;
    }


    public char[] move() throws Exception {
        // robots in the team move the same
        // others move randomly
        char[] moves = new char[robots.length];
	colors = engine.colors();
        for (int i = 0; i < robots.length; i++) {
            moves[i] = robots[i].logic.getDirection();
            robots[i].updateChaser();
            //robots[i].printChasers();
            if (robots[i].chaser() != null) {
                debug(robots[i] + " being chased by " + robots[i].chaser() + " stays put");
                moves[i] = 'Y';
            }
        }

        return moves;
    }//move

    private Robot nearestPal(int myBot) throws Exception {
        int myIndex = engine.indexOf(this);
        Robot[][] all = engine.allRobots();
        Robot me = all[myIndex][myBot];
        boolean fresh = true;
        double minDist = 0;
        int min = 0;

        for (int i = 0; i < engine.numRobots(); i++) {
            if (i != myBot) {
                //check the first time around
                if (fresh) {
                    minDist = dist(me.xpos(), me.ypos(), all[myIndex][i].xpos(), all[myIndex][i].ypos());
                    fresh = false;
                }//if
                if (dist(me.xpos(), me.ypos(), all[myIndex][i].xpos(), all[myIndex][i].ypos()) < minDist) {
                    minDist = dist(me.xpos(), me.ypos(), all[myIndex][i].xpos(), all[myIndex][i].ypos());
                    min = i;
                }//if
            }//if
        }//for
        return all[myIndex][min];
    }//nearestPal

    private Robot nearestEnemy(int myBot) throws Exception {
        //[0] is enemy player index and [1] is robot index
        int myIndex = engine.indexOf(this);
        Robot[][] all = engine.allRobots();
        Robot me = all[myIndex][myBot];
        boolean fresh = true;
        double minDist = 0;
        int[] min = new int[2];

        for (int i = 0; i < engine.numPlayers(); i++) {
            if (i != myIndex) {
                for (int j = 0; j < engine.numRobots(); j++) {
                    //check the first time around
                    if (fresh) {
                        minDist = dist(me.xpos(), me.ypos(), all[i][j].xpos(), all[i][j].ypos());
                        fresh = false;
                    }//if
                    if (dist(me.xpos(), me.ypos(), all[i][j].xpos(), all[i][j].ypos()) < minDist) {
                        minDist = dist(me.xpos(), me.ypos(), all[i][j].xpos(), all[i][j].ypos());
                        min[0] = i;
                        min[1] = j;
                    }//if
                }//for
            }//if
        }//for

        if (min[0] == myIndex)
            return null;
        else
            return all[min[0]][min[1]];
    }//nearestEnemy

    private boolean enemyOnSameSquare(int myBot) throws Exception {
        int myIndex = engine.indexOf(this);
        Robot[][] all = engine.allRobots();
        Robot me = all[myIndex][myBot];
        for (int i = 0; i < engine.numPlayers(); i++) {
            if (i != myIndex) {
                for (int j = 0; j < engine.numRobots(); j++) {
                    if (me.xpos() == all[i][j].xpos() && me.ypos() == all[i][j].ypos())
                        return true;
                }//for
            }//if
        }//for
        return false;
    }//enemyOnSameSquare

    private boolean onBoarder(int myBot) throws Exception {
        int myIndex = engine.indexOf(this);
        Robot[][] all = engine.allRobots();
        Robot me = all[myIndex][myBot];
        int size = engine.size();
        return (me.xpos() == 0 || me.xpos() == size || me.ypos() == 0 || me.ypos() == size);
    }//onBoarder

    private double dist(int x1, int y1, int x2, int y2) {
        double theX1 = (double) x1;
        double theY1 = (double) y1;
        double theX2 = (double) x2;
        double theY2 = (double) y2;
        return Math.sqrt((theX1 - theX2) * (theX1 - theX2) + (theY1 - theY2) * (theY1 - theY2));
    }//dist


    public String name() throws Exception {
        return NAME;
    }

    public Color color() throws Exception {
        return COLOR;
    }

    public boolean interactive() throws Exception {
        return false;
    }

    private static final void debug(Object o) {
        if (DEBUG)
            System.out.println(o);
    }

    private static final void debug(int i) {
        if (DEBUG)
            System.out.println(i);
    }

    private static final void print(Object o) {
        if (DEBUG)
            System.out.print(o);
    }


    /**
     * Our own robot class, which gives each robot a logic to move by
     */
    class LogicRobot extends Robot {

        Logic logic;
        Robot[] chasers = new Robot[10];
        int position = 0;

        public LogicRobot(int xpos, int ypos) throws Exception {
            super(xpos, ypos);
        }

        public void setLogic(Logic logic) {
            this.logic = logic;
        }

        public Robot chaser() throws Exception {
            Robot output = null;
            for (int i = 0; i < chasers.length; i++) {
                if (countFreq(chasers, chasers[i]) > 9) {
                    output = chasers[i];
                }//if
            }//for
            return output;
        }//chaser

        public int countFreq(Robot[] list, Robot check) {
            int count = 0;
            for (int i = 0; i < list.length; i++) {
                if (list[i] != null && check != null)
                    if (list[i]._id == check._id)
                        count++;
            }//for
            return count;
        }//maxFreq

        public int myIndex() {
            for (int i = 0; i < robots.length; i++) {
                if (robots[i] == this)
                    return i;
            }//for
            return -1;
        }//myIndex

        public void updateChaser() throws Exception {
            Robot chaser = nearestEnemy(myIndex());
            if (chaser == null)
                return;

            if (dist(chaser.xpos(), chaser.ypos(), this.xpos(), this.ypos()) < 2)
                chasers[position] = nearestEnemy(myIndex());
            else
                chasers[position] = null;
            //debug(this + " has last chaser: " + chasers[position]);
            position = (position + 1) % chasers.length;
        }//updateChaser

        public void printChasers() throws Exception {
            debug(this + " has chasers:");
            for (int i = 0; i < chasers.length; i++) {
                if (chasers[i] != null)
                    debug(chasers[i]._id);
            }//for
        }//printChasers

    }//LogicRobot

    /**
     * All logic classes need to extend this class
     */
    abstract class Logic {
        public abstract char getDirection() throws Exception;
    }

    /**
     * Keeps track of moves needed to create a set of squares
     * without going over the same edge twice
     */
    class DiagonalSquaresLogic extends Logic {

        LogicRobot robot;        // the robot running this logic
        int squareSize;
        int currentSide;    // size of current side
        int changes;        // times direction changed

        // can build squares in different directions and ways...
        static final int DIAG_DOWN = 0;
        static final int DIAG_UP = 1;
        static final int LEFT = 2;
        static final int RIGHT = 3;
        static final int UP = 4;
        static final int DOWN = 5;

        final char[] startDirections = {_CNORTH, _CSOUTH, _CNORTH, _CSOUTH, _CEAST, _CWEST};
        final char[][] directions = {
            {_CEAST, _CSOUTH, _CWEST, _CSOUTH, _CEAST, _CNORTH},
            {_CWEST, _CNORTH, _CEAST, _CNORTH, _CWEST, _CSOUTH},
            {_CWEST, _CSOUTH, _CEAST, _CSOUTH, _CWEST, _CNORTH, _CWEST, _CNORTH, _CEAST, _CNORTH, _CWEST, _CSOUTH},
            {_CEAST, _CNORTH, _CWEST, _CNORTH, _CEAST, _CSOUTH, _CEAST, _CSOUTH, _CWEST, _CSOUTH, _CWEST, _CNORTH},
            {_CNORTH, _CWEST, _CSOUTH, _CWEST, _CNORTH, _CEAST, _CNORTH, _CEAST, _CSOUTH, _CEAST, _CNORTH, _CWEST},
            {_CSOUTH, _CEAST, _CNORTH, _CEAST, _CSOUTH, _CWEST, _CSOUTH, _CWEST, _CNORTH, _CWEST, _CSOUTH, _CEAST}

        };

        char currentDirection;
        int heading;
        int numMoves;
        int movesAtLastChange;
        private int[] goTo;

        public DiagonalSquaresLogic(LogicRobot robot, int squareSize, int directionType) {
            this.robot = robot;
            this.squareSize = squareSize;
            this.heading = directionType;
            currentDirection = startDirections[directionType];
        }

        public char getDirection() throws Exception {

            if (heading == DIAG_DOWN && robot.xpos() == boardSize-1) {
                robot.logic = new SlowSquaresLogic(robot, squareSize, SlowSquaresLogic.UP);
                return robot.logic.getDirection();

            } else if (heading == DIAG_DOWN && robot.ypos() == boardSize-1) {
                robot.logic = new SlowSquaresLogic(robot, squareSize, SlowSquaresLogic.LEFT);
                return robot.logic.getDirection();
            } else if (heading == DIAG_UP && robot.xpos() == 0) {
                robot.logic = new SlowSquaresLogic(robot, squareSize, SlowSquaresLogic.DOWN);
                return robot.logic.getDirection();
            } else if (heading == DIAG_UP && robot.ypos() == 0) {
                robot.logic = new SlowSquaresLogic(robot, squareSize, SlowSquaresLogic.RIGHT);
                return robot.logic.getDirection();
            } else if (goTo != null) {
                if (robot.xpos() < goTo[0]) {
                    return _CEAST;
                } else if (robot.xpos() > goTo[0]) {
                    return _CWEST;
                } else if (robot.ypos() < goTo[1]) {
                    return _CSOUTH;
                } else if (robot.ypos() > goTo[1]) {
                    return _CNORTH;
                } else {
                    goTo = null;
                }
            } else if (currentSide > squareSize) {
                currentSide = 0;
                currentDirection = directions[heading][changes % directions[heading].length];
                changes++;
            }

            currentSide++;
            //print(currentDirection + ",");
            return currentDirection;

        }

        public void goTo(int x, int y) {

            x = Math.max(0, x);
            x = Math.min(boardSize - 1, x);

            y = Math.max(0, y);
            y = Math.max(boardSize - 1, y);

            goTo = new int[]{x, y};
        }
    }

    class SlowSquaresLogic extends Logic {

        Robot robot;

        static final int UP = 0;
        static final int DOWN = 1;
        static final int LEFT = 2;
        static final int RIGHT = 3;
        final char[][] directions = {{ _CNORTH, _CWEST, _CSOUTH, _CEAST},
                                     { _CSOUTH, _CEAST, _CNORTH, _CWEST},
                                     { _CWEST, _CSOUTH, _CEAST, _CNORTH},
                                     { _CEAST, _CSOUTH, _CWEST, _CNORTH}};
        char direction;
        int heading;
        int currentLength, changes = 1;
        int[] lastLength = new int[2];  // holds the last vertical and horizontal lengths
        private int sideLength;
        private int[] goTo;
        private boolean changedHeading;

        public SlowSquaresLogic(Robot robot, int sideLength, int heading) {
            this.robot = robot;
            this.sideLength = sideLength;
            this.heading = heading;
            lastLength[0] = lastLength[1] = sideLength;
            direction = directions[heading][0];
        }

        public char getDirection() throws Exception {

            char nextDirection = direction;
            if (goTo != null) {
                if (robot.xpos() < goTo[0]) {
                    nextDirection = _CEAST;
                } else if (robot.xpos() > goTo[0]) {
                    nextDirection = _CWEST;
                } else if (robot.ypos() < goTo[1]) {
                    nextDirection = _CSOUTH;
                } else if (robot.ypos() > goTo[1]) {
                    nextDirection = _CNORTH;
                } else {
                    goTo = null;
                }
            } else if (changedHeading) {

                currentLength = 0;
                changes = 1;
                nextDirection = directions[heading][0];
                changedHeading = false;

            } else if (currentLength > lastLength[changes % 2]) {

                // check for when we complete a square
                if (heading == UP && changes == directions[heading].length) {
                    goTo(robot.xpos(), robot.ypos() - (lastLength[1]+sideLength));
                    nextDirection = directions[heading][0];
                } else if (heading == DOWN && changes == directions[heading].length) {
                    goTo(robot.xpos(), robot.ypos() + (lastLength[1] + sideLength));
                    nextDirection = directions[heading][0];
                } else if (heading == LEFT && changes == directions[heading].length) {
                    goTo(robot.xpos() - (lastLength[1]+sideLength), robot.ypos());
                    nextDirection = directions[heading][0];
                } else if (heading == RIGHT && changes == directions[heading].length) {
                    goTo(robot.xpos() + (lastLength[1]+sideLength), robot.ypos());
                    nextDirection = directions[heading][0];
                } else {
                    lastLength[changes % 2] = currentLength;
                    lastLength[(changes + 1) % 2] = sideLength;
                    currentLength = 0;
                    nextDirection = directions[heading][changes % directions[heading].length];
                    changes++;
                }
            }


            int nextX = robot.xpos();
            int nextY = robot.ypos();

            switch (nextDirection) {
                case _CNORTH:
                    nextY--;
                    break;
                case _CWEST:
                    nextX--;
                    break;
                case _CSOUTH:
                    nextY++;
                    break;
                case _CEAST:
                    nextX++;
                    break;
            }


            if (nextY < 0) {
                // turn around and go left
                debug("robot " + robot.ID() + " going left");
                goTo(nextX - sideLength, 0);
                heading = LEFT;
                changedHeading = true;


            } else if (nextY > boardSize-1) {
                // turn around and go up
                debug("robot " + robot.ID() + " going up");
                goTo(nextX - (2*sideLength+2), nextY-1);
                heading = UP;
                changedHeading = true;

            }
            else if (nextX < 0) {
                debug("robot " + robot.ID() + " going right");
                // go right
                goTo(0, nextY - sideLength);
                heading = RIGHT;
                changedHeading = true;

            }
            else if (nextX > boardSize-1) {
                debug("robot " + robot.ID() + " going left");
                // go left
                goTo(nextX-1, nextY - sideLength);
                heading = LEFT;
                changedHeading = true;

            }

            // check that the square we are about to turn into is not filled
            /*else if (nextX < boardSize && nextY < boardSize && (!engine.filled()[nextX][nextY] || changes > 2)) {
                // ok to turn here
                direction = nextDirection;
            } else if (nextX > boardSize || nextY > boardSize) {
                // change direction!
            }*/

            direction = nextDirection;
            currentLength++;
            return direction;
        }

        private void goTo(int x, int y) throws Exception {

            x = Math.max(-1, x);
            x = Math.min(boardSize, x);

            y = Math.max(-1, y);
            y = Math.min(boardSize, y);

            goTo = new int[]{x, y};
            changes = 1;
            lastLength[0] = lastLength[1] = sideLength;
            currentLength = 0;
            debug("robot " + robot.ID() + " going to [" + x + "," + y + "]");
        }

    }


    public static final char[] DISRUPT_directions = new char[]
	{ 'N', 'E', 'S', 'W' };
    
    public static final int[][] DISRUPT_vectors = new int[][]
	{ {0, -1}, {1, 0}, {0, 1}, {-1, 0} };

    public static final boolean DISRUPT_debug = false;

    /**
     * This logic tries to disrupt other players.
     */
    class DisruptLogic extends Logic {

	private LogicRobot robot;
	private int lastDirection;

        public DisruptLogic(LogicRobot robot) {
            this.robot = robot;
	    lastDirection = DISRUPT_directions[random.nextInt(4)];
        }

        public char getDirection() throws Exception {
	    int newDirection = lastDirection;
	    int bestQuality = -100;
	    for(int test = lastDirection; test < lastDirection + 4; ++test) {
		int q = quality(test & 0x03);
		if (q > bestQuality) {
		    newDirection = test & 0x03;
		    bestQuality = q;
		}
	    }
	    lastDirection = newDirection;
	    if (DISRUPT_debug) {
		System.err.println("picking " + DISRUPT_directions[lastDirection]);
	    }

	    return DISRUPT_directions[lastDirection];
        }

	private int quality(int direction) throws Exception {
	    int x = robot.xpos(), y = robot.ypos();
	    x += DISRUPT_vectors[direction][0];
	    y += DISRUPT_vectors[direction][1];
	    if (x < 0 || x >= boardSize || y < 0 || y >= boardSize)
		return -1; // never pick this!
	    int color = colors[x][y];
	    if (DISRUPT_debug) {
		System.err.println("disrupter nr. " + robot.myIndex() + " direction " + DISRUPT_directions[direction] + " to (" + x + "," + y + ")" + " is " + color);
	    }
	    switch(color) {
	    case _CFILLED:
		return 0;
	    case _CEMPTY:
	    case _CCONTESTED:
		return 2;
	    }
	    if (color == engine.indexOf(OldGroup6Player3.this))
		return 1;
	    else
		return 3;
	}
    }

}
