/* $Id$
 * Group7Player6: Player by Eva & Cheoljoo
 *
 * Group members: Eva, Jason, Cheoljoo
 */
package Rectangles;

import java.util.*;
import java.io.*;
import java.awt.*;
import ui.*;

public final class OldGroup7Player6 implements IFCPlayer {
  Rectangles rectangles;

  static Random random;
  static final String _CNAME = "PlaidPlayer";
  static final Color _CCOLOR = new Color(1.0f, 0.5f, 0.0f);
  static final int _NORTH = 0;
  static final int _SOUTH = 1;
  static final int _WEST = 2;
  static final int _EAST = 3;

  Robot[] robots;        /* robots */
  int numRobots;         /* number of robots */
  int size;              /* board size */
  int destX[], destY[];  /* destinations of robots; -1 for grid walking mode */
  int direction[];       /* robot directions; one of _NORTH, ... */

  public Robot[] register(Rectangles rectangles) throws Exception {
    this.rectangles = rectangles;
    this.numRobots = rectangles.numRobots();
    this.robots = new Robot[numRobots];
    this.size = rectangles.size();
    this.destX = new int[numRobots];
    this.destY = new int[numRobots];
    this.direction = new int[numRobots];
    this.random = java.security.SecureRandom.getInstance("SHA1PRNG");

    /* determines the initial locations of the robots */
    int x = 0;
    int y = 0; 
    int inc = (size/numRobots) * 2;
    for (int i = 0; i < numRobots/2; i++) {
      robots[i] = new Robot(x, size - 1);
      x += inc;
      destX[i] = x - 1;
      destY[i] = 0;
    }
    for (int i = numRobots/2; i < numRobots; i++) {
      robots[i] = new Robot(size - 1, y);
      y += inc;
      destX[i] = 0;
      destY[i] = y - 1;
    }

    /* if the number of robots are less than 3, then skip the plaid walking
       and do the grid walking */
    if (numRobots < 5)
      for (int i = 0; i < numRobots; i++)
	destX[i] = destY[i] = -1;

    return robots;
  }

  public String name() throws Exception {
    return _CNAME;
  }

  public Color color() throws Exception {
    return _CCOLOR;
  }

  public boolean interactive() throws Exception {
    return false;
  }

  /* determine the next moves of robots */
  public char[] move() throws Exception {
    char[] moves = new char[numRobots];
    
    for (int i = 0; i < numRobots; i++) {
      moves[i] = nextMove(i);
      direction[i] = charDir(moves[i]);
    }

    return moves;
  }

  /* determine the next move of the i-th robot */
  public char nextMove(int i) throws Exception {
    char move = 'S';
    int x = robots[i].xpos();  /* current x-pos of the i-th robot */
    int y = robots[i].ypos();  /* current y-pos of the i-th robot */
    int destx = destX[i];      /* destination x-pos of the i-th robot */
    int desty = destY[i];      /* destination y-pos of the i-th robot */

    /* if grid walking mode */
    if (destX[i] == -1 || destY[i] == -1) {
      int validCont = validMoveDir(x, y, direction[i]);
      int rnd = random.nextInt(15);
      if (numRobots < 5)
	rnd = random.nextInt(10);
      if (validCont > 0) {
	int validRight = validMoveDir(x, y, nextDirection(direction[i])); 
	int validLeft = 
	  validMoveDir(x, y, 
		       nextDirection(
			 nextDirection(
			   nextDirection(direction[i]))));
	if (validRight > 0 && rnd < 3) {
	  if (validLeft > 0 && rnd == 0)
	    return dirChar(nextDirection(
			     nextDirection(
			       nextDirection(direction[i]))));
	  else
	    return dirChar(nextDirection(direction[i]));
	}
	else
	  return dirChar(direction[i]);
      }
      else if (validCont == 0) /* out of range */
	return dirChar(nextDirection(direction[i]));
      else if (validCont == _CFILLED)
	return dirChar(nextDirection(direction[i]));
    }
    
    if (x > destx) /* destx <-- x */
      move = 'W';
    else if (x < destx) /* x --> destx */
      move = 'E';
    else if (x == destx) {
      /* now move along y-axis */
      if (y > desty) /* desty --> y */
	move = 'N';
      else if (desty > y)
	move = 'S';
      else if (y == desty) {
	/* now arrived at the destination; get new destination */
	destX[i] = random.nextInt(size);
	destY[i] = random.nextInt(size);
	destY[i] = -1;
	return nextMove(i);
      }
    }
    return move;
  }

  /* checks if moving to the given direction from [x, y] is valid */
  public int validMoveDir(int x, int y, int dir) throws Exception {
    switch (dir) {
    case  _EAST: return validMove(x + 1, y);
    case  _WEST: return validMove(x - 1, y);
    case _SOUTH: return validMove(x, y + 1);
    case _NORTH: return validMove(x, y - 1);
    default: return 1;
    }
  }

  /* checks if moving to [x, y] is valid; returns _CFILLED (-1) if the
   * position is already filled with black paint, returns 0 if it's
   * out of range, and returns 1 if it's ok 
   */
  public int validMove(int x, int y) throws Exception {
    int[][] col = rectangles.colors();
    if (((x >= 0) && (x <= size-1)) && ((y >= 0) && (y <= size-1))) {
      if (col[x][y] == _CFILLED)
	return _CFILLED;
      else 
	return 1;
    }
    else
      return 0;
  }

  /* returns a random direction */
  public int randomDirection() {
    int dir = random.nextInt(4);
    switch (dir) {
    case 0: return _EAST;
    case 1: return _WEST;
    case 2: return _SOUTH;
    case 3: return _NORTH;
    default: return _SOUTH;
    }
  }
  
  /* returns the 'char' of the given direction */
  public char dirChar(int dir) {
    switch (dir) {
    case  _EAST: return 'E';
    case  _WEST: return 'W';
    case _SOUTH: return 'S';
    case _NORTH: return 'N';
    default: return 'S';
    }
  }

  /* inverse function of dirChar */
  public int charDir(char ch) {
    switch (ch) {
    case 'E': return _EAST;
    case 'W': return _WEST;
    case 'S': return _SOUTH;
    case 'N': return _NORTH;
    default: return _EAST;
    }
  }

  /* returns the next direction in grid walking */
  public int nextDirection(int dir) {
    switch (dir) {
    case  _EAST: return _SOUTH;
    case  _WEST: return _NORTH;
    case _SOUTH: return _WEST;
    case _NORTH: return _EAST;
    default: return _EAST;
    }
  }

  /* determine if the given direction (_NORTH, _SOUTH, ...) of [x, y] 
   * is filled or not 
   */
  public boolean filled(int x, int y, int direction) throws Exception {
    int[][] colors = rectangles.colors();

    switch (direction) {
    case _EAST:
      return (x + 1 < size) ? (colors[x + 1][y] == _CFILLED) : false;
    case _WEST:
      return (x - 1 >= 0) ? (colors[x - 1][y] == _CFILLED) : false;
    case _SOUTH:
      return (y + 1 < size) ? (colors[x][y + 1] == _CFILLED) : false;
    case _NORTH:
      return (y - 1 >= 0) ? (colors[x][y - 1] == _CFILLED) : false;
    default:
      return true;
    }
  }

  /* Manhattan distance between two points */
  public int distance(int x, int y, int dx, int dy) {
    return Math.abs(x - dx) + Math.abs(y - dy);
  }
}
